library flutter_backend;

import 'package:mysql1/mysql1.dart';

bool isDoubleQuotesFields(String fieldType) {
  switch (fieldType) {
    case 'VARCHAR':
      return true;
      break;
    case 'VAR_STRING':
      return true;
      break;
    case 'STRING':
      return true;
      break;
    case "TIMESTAMP":
      return true;
      break;
    case "TIME":
      return true;
      break;
    case 'DATE':
      return true;
      break;
    default:
      return false;
      break;
  }
}

class DB {
  getWhereString(List where) {
    String whereString = "";
    if (where.isEmpty) {
      whereString = "";
    } else {
      whereString = " WHERE ";
      where.forEach((statement) {
        whereString += statement;
      });
    }
    return whereString;
  }

  getFieldDefinition(Map field) {
    List fields = [];
    field.forEach((key, value) {
      fields.add(key);
    });
    return fields.join(",");
  }

  getValueDefinition(Map field) {
    List fields = [];
    field.forEach((key, value) {
      fields.add("'$value'");
    });
    return fields.join(",");
  }

  getUpdateDefinition(Map field) {
    List fields = [];
    field.forEach((key, value) {
      fields.add("$key = '$value'");
    });
    return fields.join(",");
  }

  Future<List<Map>> get({
    String table,
    List where = const [],
  }) async {
    var whereString = getWhereString(where);

    var db = DatabaseConection();
    var query = "select * from $table $whereString";
    var values = [];
    var results = await db.query(query, values);

    var fieldType = [];
    for (Field field in results.fields) {
      fieldType.add({
        "field_name": field.name,
        "type": field.typeString,
      });
    }

    List<Map> rowList = [];
    for (var row in results) {
      Map rowMap = {};
      for (var i = 0; i < row.length; i++) {
        var fieldName = fieldType[i]["field_name"];
        var type = fieldType[i]["type"];
        var isString = isDoubleQuotesFields(type);
        var value = isString ? "${row[i]}" : row[i];
        // var value = "${row[i]}";

        rowMap[fieldName] = value;
      }
      rowList.add(rowMap);
    }

    return rowList;
  }

  create({
    String table,
    Map field,
  }) async {
    // var whereString = getWhereString(where);

    var fieldDefinition = getFieldDefinition(field);
    var valueDefinition = getValueDefinition(field);

    var db = DatabaseConection();
    var query =
        "insert into $table ($fieldDefinition) values($valueDefinition)";
    var values = [];
    var results = await db.query(query, values);

    print(query);
    return results;
  }

  update({
    String table,
    Map field,
    List where = const [],
  }) async {
    var whereString = getWhereString(where);

    var updateDefinition = getUpdateDefinition(field);

    var db = DatabaseConection();
    var query = "update $table SET $updateDefinition $whereString";
    var values = [];
    var results = await db.query(query, values);

    print(query);
    return results;
  }

  delete({
    String table,
    List where = const [],
  }) async {
    var whereString = getWhereString(where);

    var db = DatabaseConection();
    var query = "delete from $table $whereString";
    var values = [];
    var results = await db.query(query, values);

    print(query);
    return results;
  }
}

class DatabaseConection {
  var connectionSetting = ConnectionSettings(
    host: '127.0.0.1',
    port: 3306,
    user: 'root',
    // password: '', //comment it if u dont use any password
    db: 'flutter_db',
  );

  MySqlConnection conn;
  connect() async {
    conn = await MySqlConnection.connect(connectionSetting);
  }

  close() async {
    await conn.close();
  }

  query(String query, List values) async {
    await connect();
    var results = await conn.query(query, values);
    await close();

    return results;
  }
}
