import '../controller/controller.dart';
import '../request_helper/request_helper.dart';

class RouteItem {
  final String endpoint;
  final String method;
  final Future<Null> Function(RequestHelper rh) callBackFunction;

  RouteItem({
    this.endpoint,
    this.method,
    this.callBackFunction,
  });
}

List<RouteItem> routeList = [];

class Route {
  static createRoute({
    String endpoint,
    String method,
    dynamic callBackFunction,
  }) {
    print("endpoint: $endpoint");
    print("method: $method");
    print("---");

    routeList.add(
      RouteItem(
        endpoint: endpoint,
        method: method,
        callBackFunction: callBackFunction,
      ),
    );
  }

  static get(String endpoint, dynamic callBackFunction) {
    createRoute(
      endpoint: endpoint,
      method: "GET",
      callBackFunction: callBackFunction,
    );
  }

  static post(String endpoint, dynamic callBackFunction) {
    createRoute(
      endpoint: endpoint,
      method: "POST",
      callBackFunction: callBackFunction,
    );
  }

  static put(String endpoint, dynamic callBackFunction) {
    createRoute(
      endpoint: endpoint,
      method: "PUT",
      callBackFunction: callBackFunction,
    );
  }

  static delete(String endpoint, dynamic callBackFunction) {
    createRoute(
      endpoint: endpoint,
      method: "DELETE",
      callBackFunction: callBackFunction,
    );
  }

  static create({
    String endpoint,
    Controller controller,
  }) {
    Route.get("$endpoint", (RequestHelper rh) async {
      Controller c = controller.createInstance();
      c.bind(rh);
      c.index();
    });

    Route.get("$endpoint/:id", (RequestHelper rh) {
      Controller c = controller.createInstance();
      c.bind(rh);
      c.getSingleData();
    });

    Route.post("$endpoint", (RequestHelper rh) {
      Controller c = controller.createInstance();
      c.bind(rh);
      c.create();
    });

    Route.put("$endpoint/:id", (RequestHelper rh) {
      Controller c = controller.createInstance();
      c.bind(rh);
      c.update();
    });

    Route.delete("$endpoint/:id", (RequestHelper rh) {
      Controller c = controller.createInstance();
      c.bind(rh);
      c.delete();
    });
  }
}
