import 'dart:convert';
import 'dart:io';

import 'package:flutter_backend/shared/helper/route/route.dart';

import '../request_helper/request_helper.dart';

String _host = InternetAddress.loopbackIPv4.host;

parseRouteName(String endpoint) {
  var availableQueryParameters = [];
  var arr = endpoint.split("/");
  for (var i = 0; i < arr.length; i++) {
    var ep = arr[i];
    var firstChar = ep[0];

    if (firstChar == ":") {
      var param = ep.substring(1);
      availableQueryParameters.add(param);
      arr[i] = ":param";
    } else if (RegExp(r'^[0-9]+$').hasMatch(firstChar)) {
      arr[i] = ":param";
    }
  }
  return arr.join("/");
}

List<Map<String, dynamic>> getQueryParameters(
    String endpoint, String routeName) {
  List<Map<String, dynamic>> parameters = [];

  var routeNameArr = routeName.split("/");
  var endpointArr = endpoint.split("/");

  for (var i = 0; i < routeNameArr.length; i++) {
    var ep = routeNameArr[i];
    var firstChar = ep[0];

    if (firstChar == ":") {
      var param = ep.substring(1);
      parameters.add({
        "key": param,
        "value": null,
      });
    }
  }

  var paramIndex = 0;
  for (var i = 0; i < endpointArr.length; i++) {
    var ep = endpointArr[i];
    var firstChar = ep[0];

    if (RegExp(r'^[0-9]+$').hasMatch(firstChar)) {
      parameters[paramIndex]["value"] = ep;
      paramIndex++;
    }
  }

  return parameters;
}

class Server {
  static runServer({
    Function initRouterFunction,
  }) async {
    var server = await HttpServer.bind(_host, 8080);
    initRouterFunction();
    await for (var req in server) {
      HttpResponse res = req.response;

      var endpoint = req.uri.toString().substring(1);

      if (endpoint.contains("?")) {
        endpoint = endpoint.split("?")[0];
      }

      var routeItemList = routeList
          .where((routeItem) =>
              parseRouteName(routeItem.endpoint) == parseRouteName(endpoint) &&
              routeItem.method == req.method)
          .toList();

      if (routeItemList.isEmpty) {
        try {
          res
            ..statusCode = HttpStatus.badRequest
            ..headers.contentType = ContentType.json
            ..write(jsonEncode(
              {
                "error": true,
                "message": "This endpoint is not registered",
                "endpoint": endpoint,
                "method": req.method,
              },
            ));
          res.close();
        } catch (e) {
          res
            ..statusCode = HttpStatus.internalServerError
            ..write('Exception during file I/O: $e.');
          res.close();
        }
      } else {
        RouteItem routeItem = routeItemList.first;

        var parameters = getQueryParameters(endpoint, routeItem.endpoint);
        var postData;

        try {
          String content = await utf8.decoder.bind(req).join();

          try {
            postData = jsonDecode(content) as Map;
          } catch (_) {
            postData = {};
            print("decodeError");
          }

          var requestHelper = RequestHelper(
            req: req,
            res: res,
            postData: postData,
            parameters: parameters,
          );

          routeItem.callBackFunction(requestHelper);
        } catch (e) {
          res
            ..statusCode = HttpStatus.internalServerError
            ..write('Exception during file I/O: $e.');
          res.close();
        }
      }
    }
  }
}
